/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field.shape;

import com.vividsolutions.jts.geom.Geometry;
import javax.swing.*;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.field.model.Shape;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureIterator;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.opengis.feature.simple.SimpleFeature;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;
import org.openide.windows.TopComponent;

@MultiViewElement.Registration(displayName = "#LBL_Shape_VISUAL",
iconBase = "nl/cloudfarming/client/field/shape/file_extension_shp.png",
mimeType = ShapeDataObject.MIMETYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "ShapeVisual",
position = 2000)
@Messages("LBL_Shape_VISUAL=Visual")
public final class ShapeVisualElement extends JPanel implements MultiViewElement {

    private ShapeDataObject obj;
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private final JLabel loading = new JLabel("Loading...");

    public ShapeVisualElement(Lookup lkp) {
        obj = lkp.lookup(ShapeDataObject.class);
        assert obj != null;
        initComponents();
        contentPanel.setLayout(new MigLayout("wrap 1"));
        contentPanel.add(loading);
        Loader loader = new Loader();
        loader.execute();
    }

    @Override
    public String getName() {
        return "ShapeVisualElement";
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        contentPanel = new javax.swing.JPanel();

        javax.swing.GroupLayout contentPanelLayout = new javax.swing.GroupLayout(contentPanel);
        contentPanel.setLayout(contentPanelLayout);
        contentPanelLayout.setHorizontalGroup(
            contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 394, Short.MAX_VALUE)
        );
        contentPanelLayout.setVerticalGroup(
            contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 294, Short.MAX_VALUE)
        );

        jScrollPane1.setViewportView(contentPanel);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel contentPanel;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
        callback.updateTitle(obj.getName());
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private final class Loader extends SwingWorker<Void, Void> {

        public Loader() {
        }

        @Override
        protected Void doInBackground() throws Exception {
            FileObject fileObject = ShapeVisualElement.this.obj.getPrimaryFile();
            ShapefileDataStore shpDataStore = new ShapefileDataStore(fileObject.getURL());
            ShapeVisualElement.this.contentPanel.add(new JLabel(fileObject.getURL().toString()));
            String typeName = shpDataStore.getTypeNames()[0];
            FeatureSource featureSource = shpDataStore.getFeatureSource(typeName);

            FeatureCollection result = featureSource.getFeatures();
            FeatureIterator iterator = result.features();
            while (iterator.hasNext()) {
                SimpleFeature feature = (SimpleFeature) iterator.next();
                Shape shape = parseShape(feature);
                // Debug: LOG.info("Geometrytype: " + feature.getType().getTypeName());
                ShapeVisualElement.this.contentPanel.add(new JLabel("+" + shape.getName() + "-" + shape.getGeometry().toText()));
            }
            iterator.close();
            ShapeVisualElement.this.contentPanel.remove(loading);
            ShapeVisualElement.this.contentPanel.repaint();

            return null;
        }

        /**
         *
         * @param payload
         * @return
         */
        public Shape parseShape(SimpleFeature payload) {

            int attIndexPlotPoly = payload.getFeatureType().indexOf("the_geom");
            int attIndexPlotName = payload.getFeatureType().indexOf("KAVEL");
            Geometry geometry = null;
            if (attIndexPlotPoly >= 0) {
                geometry = (Geometry) payload.getAttribute(attIndexPlotPoly);
            }
            String shapeName = "Unknown";
            if (attIndexPlotName >= 0) {
                shapeName = (String) payload.getAttribute(attIndexPlotName);
            }

            Shape shape = new Shape(geometry);
            shape.setName(shapeName);
            return shape;
        }
    }
}
